@@ -41,6 +41,7 @@ static void     gail_finish_select       (GtkWidget            *widget);
 static void     gail_map_cb              (GtkWidget            *widget);
 static gint     gail_focus_idle_handler  (gpointer             data);
 static void     gail_focus_notify        (GtkWidget            *widget);
+static void     gail_focus_notify_when_idle (GtkWidget            *widget);
 
 static void     gail_focus_tracker_init ();
 
@@ -98,9 +99,17 @@ gail_get_accessible_for_widget (GtkWidget *widget,
     } 
   else if (GTK_IS_NOTEBOOK (widget)) 
     {
+      GtkNotebook *notebook;
       gint page_num;
 
-      page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
+      notebook = GTK_WIDGET (widget);
+      /*
+       * Report the currently focused tab rather than the currently selected tab
+       */
+      if (notebook->focus_tab)
+        {
+          page_num = g_list_index (notebook->children, notebook->focus_tab->data);
+        }
       if (page_num != -1)
         {
           obj = gtk_widget_get_accessible (widget);
@@ -223,11 +232,7 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
   /*
    * The widget may not yet be visible on the screen so we wait until it is.
    */
-  if (focus_notify_handler)
-    {
-      gtk_idle_remove (focus_notify_handler);
-    }
-  focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
+  gail_focus_notify_when_idle (widget);
   return TRUE; 
 }
 
@@ -266,20 +271,17 @@ gail_finish_select (GtkWidget *widget)
    */
     return;
   
-  if (focus_notify_handler)
-    {
-      gtk_idle_remove (focus_notify_handler);
-      focus_notify_handler = 0;
-    }
   /*
    * If previously focused widget is not a GtkMenuItem or a GtkMenu,
    * keep track of it so we can return to it after menubar is deactivated
    */
-  if (!GTK_IS_MENU_ITEM (focus_widget)  && !GTK_IS_MENU (focus_widget))
+  if (focus_widget && !GTK_IS_MENU_ITEM (focus_widget)  && !GTK_IS_MENU (focus_widget))
     {
       focus_before_menu = focus_widget;
+      g_object_add_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+
     } 
-  gail_focus_notify (widget);
+  gail_focus_notify_when_idle (widget);
 
   return; 
 }
@@ -322,7 +324,7 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
           active_menu_item = GTK_MENU_SHELL (parent_menu_shell)->active_menu_item;
           if (active_menu_item)
             {
-              focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, active_menu_item);
+              gail_focus_notify_when_idle (active_menu_item);
             }
         }
     }
@@ -344,15 +346,24 @@ gail_focus_notify (GtkWidget *widget)
   AtkObject *atk_obj;
   gboolean transient;
 
-  focus_widget = widget;
-  if (focus_before_menu == focus_widget)
+  if (widget != focus_widget)
     {
+      if (focus_widget)
+        {
+          g_object_remove_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
+        }
+      focus_widget = widget;
+      g_object_add_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
       /*
        * The UI may not have been updated yet; e.g. in gtkhtml2
        * html_view_layout() is called in a idle handler
        */
-      focus_before_menu = NULL;
-      focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, focus_widget);
+      if (focus_widget == focus_before_menu)
+        {
+          g_object_remove_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+          focus_before_menu = NULL;
+        }
+      gail_focus_notify_when_idle (focus_widget);
     }
   else
     {
@@ -363,6 +374,23 @@ gail_focus_notify (GtkWidget *widget)
     }
 }
 
+static void
+gail_focus_notify_when_idle (GtkWidget *widget)
+{
+  if (focus_notify_handler)
+    {
+      gtk_idle_remove (focus_notify_handler);
+    }
+
+  if (widget)
+    {
+      focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
+    }
+  else
+    {
+      focus_notify_handler = 0;
+    }
+}
 static gboolean
 gail_deactivate_watcher (GSignalInvocationHint *ihint,
                     guint                  n_param_values,
@@ -372,25 +400,18 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
   GObject *object;
   GtkWidget *widget;
   GtkMenuShell *shell;
+  GtkWidget *focus = NULL;
 
   object = g_value_get_object (param_values + 0);
   g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
   widget = GTK_WIDGET (object);
 
-  if (focus_notify_handler)
-    {
-      gtk_idle_remove (focus_notify_handler);
-      focus_notify_handler = 0;
-    }
   g_return_val_if_fail (GTK_IS_MENU_SHELL(widget), TRUE);
   shell = GTK_MENU_SHELL(widget);
-  if (shell->parent_menu_shell)
-    return TRUE;
-
-  if (focus_before_menu)
-  {
-    focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, focus_before_menu);
-  }
+  if (!shell->parent_menu_shell)
+    focus = focus_before_menu;
+      
+  gail_focus_notify_when_idle (focus);
 
   return TRUE; 
 }
