@@ -584,7 +584,9 @@ get_image_from_button (GtkWidget *button)
     child = gtk_bin_get_child (GTK_BIN (child));
   if (GTK_IS_BOX (child))
     {
-      list = gtk_container_children (GTK_CONTAINER (child));
+      list = gtk_container_get_children (GTK_CONTAINER (child));
+      if (!list)
+        return NULL;
       if (GTK_IS_IMAGE (list->data))
         image = GTK_IMAGE (list->data);
       g_list_free (list);
@@ -631,12 +633,14 @@ gail_button_get_image_size (AtkImage *image,
   widget = GTK_ACCESSIBLE (image)->widget;
 
   if (widget == NULL)
+    {
     /*
      * State is defunct
      */
-    *width = -1;
-    *height = -1;
-    return;
+      *width = -1;
+      *height = -1;
+      return;
+    }
 
   button_image = get_image_from_button (widget);
 
@@ -706,7 +710,7 @@ get_label_from_button (GtkWidget *button)
        */
       GList *children, *tmp_list;
  
-      children = gtk_container_children (GTK_CONTAINER (child));
+      children = gtk_container_get_children (GTK_CONTAINER (child));
 
       child = NULL;
       for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
