@@ -20,13 +20,10 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gailframe.h"
-#include "gailframefactory.h"
 
 static void                  gail_frame_class_init       (GailFrameClass  *klass);
 static G_CONST_RETURN gchar* gail_frame_get_name         (AtkObject       *obj);
 
-static GailContainer* parent_class = NULL;
-
 GType
 gail_frame_get_type (void)
 {
@@ -59,11 +56,6 @@ static void
 gail_frame_class_init (GailFrameClass *klass)
 {
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
-
-  widget_class = (GailWidgetClass*)klass;
-
-  parent_class = g_type_class_ref (GAIL_TYPE_CONTAINER);
 
   class->get_name = gail_frame_get_name;
 }
@@ -78,9 +70,9 @@ gail_frame_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_FRAME, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   accessible->role = ATK_ROLE_PANEL;
 
   return accessible;
