@@ -172,6 +172,8 @@ gail_notebook_ref_child (AtkObject      *obj,
      * See bug 58674
      */
       accessible = gail_notebook_page_new (gtk_notebook, i);
+      if (!accessible)
+        return NULL;
       gail_notebook->page_cache = g_list_append (gail_notebook->page_cache, 
                                                  accessible);
       g_return_val_if_fail (accessible, NULL);
@@ -255,18 +257,21 @@ gail_notebook_real_notify_gtk (GObject           *obj,
                                               ATK_STATE_SELECTED,
                                               TRUE);
               /*
-               * The page which is being displayed has changed so tell the focus
-               * tracker
+               * The page which is being displayed has changed but there is
+               * no need to tell the focus tracker as the focus page will also 
+               * change or a widget in the page will receive focus if the
+               * Notebook does not have tabs.
                */
-              atk_focus_tracker_notify (obj);
             }
         }
-      else if (focus_page_num != old_focus_page_num)
+      if (focus_page_num != old_focus_page_num)
         {
           AtkObject *obj;
 
-          obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
-          atk_focus_tracker_notify (obj);
+          if (gtk_notebook_get_show_tabs (gtk_notebook)) {
+            obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
+            atk_focus_tracker_notify (obj);
+          }
         }
     }
   else
