@@ -21,12 +21,14 @@
 #include "gailtextcell.h"
 #include "gailcontainercell.h"
 #include "gailcellparent.h"
-#include <util/gailmisc.h>
+#include <libgail-util/gailmisc.h>
 
 static void      gail_text_cell_class_init		(GailTextCellClass *klass);
 static void      gail_text_cell_init			(GailTextCell	*text_cell);
 static void      gail_text_cell_finalize		(GObject	*object);
 
+static G_CONST_RETURN gchar* gail_text_cell_get_name    (AtkObject      *atk_obj);
+
 static void      atk_text_interface_init		(AtkTextIface	*iface);
 
 /* atktext.h */
@@ -165,12 +167,15 @@ static void
 gail_text_cell_class_init (GailTextCellClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  AtkObjectClass *atk_object_class = ATK_OBJECT_CLASS (klass);
   GailRendererCellClass *renderer_cell_class = GAIL_RENDERER_CELL_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
   renderer_cell_class->update_cache = gail_text_cell_update_cache;
   renderer_cell_class->property_list = gail_text_cell_property_list;
 
+  atk_object_class->get_name = gail_text_cell_get_name;
+
   gobject_class->finalize = gail_text_cell_finalize;
 }
 
@@ -220,11 +225,25 @@ gail_text_cell_finalize (GObject            *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static G_CONST_RETURN gchar*
+gail_text_cell_get_name (AtkObject *atk_obj)
+{
+  if (atk_obj->name)
+    return atk_obj->name;
+  else
+    {
+      GailTextCell *text_cell = GAIL_TEXT_CELL (atk_obj);
+
+      return text_cell->cell_text;
+    }
+}
+
 static gboolean
 gail_text_cell_update_cache (GailRendererCell *cell,
                              gboolean         emit_change_signal)
 {
   GailTextCell *text_cell = GAIL_TEXT_CELL (cell);
+  AtkObject *obj = ATK_OBJECT (cell);
   gboolean rv = FALSE;
   gint temp_length;
   gchar *new_cache;
@@ -244,7 +263,12 @@ gail_text_cell_update_cache (GailRendererCell *cell,
           temp_length = text_cell->cell_length;
           text_cell->cell_text = NULL;
           text_cell->cell_length = 0;
-          g_signal_emit_by_name (cell, "text_changed::delete", 0, temp_length);
+          if (emit_change_signal)
+            {
+              g_signal_emit_by_name (cell, "text_changed::delete", 0, temp_length);
+              if (obj->name == NULL)
+                g_object_notify (G_OBJECT (obj), "accessible-name");
+            }
         }
     }
   else
@@ -267,9 +291,16 @@ gail_text_cell_update_cache (GailRendererCell *cell,
   gail_text_util_text_setup (text_cell->textutil, text_cell->cell_text);
   
   if (rv)
-    g_signal_emit_by_name (cell, "text_changed::insert",
-			   0, text_cell->cell_length);
+    {
+      if (emit_change_signal)
+        {
+          g_signal_emit_by_name (cell, "text_changed::insert",
+  			         0, text_cell->cell_length);
 
+          if (obj->name == NULL)
+            g_object_notify (G_OBJECT (obj), "accessible-name");
+        }
+    }
   return rv;
 }
 
@@ -298,7 +329,7 @@ gail_text_cell_get_text (AtkText *text,
 {
   if (GAIL_TEXT_CELL (text)->cell_text)
     return gail_text_util_get_substring (GAIL_TEXT_CELL (text)->textutil,
-              &start_pos, &end_pos);
+              start_pos, end_pos);
   else
     return g_strdup ("");
 }
