@@ -21,7 +21,7 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include "gailtextview.h"
-#include <util/gailmisc.h>
+#include <libgail-util/gailmisc.h>
 
 static void       gail_text_view_class_init            (GailTextViewClass *klass);
 static void       gail_text_view_init                  (GailTextView      *text_view);
@@ -448,6 +448,7 @@ gail_text_view_get_character_at_offset (AtkText *text,
   GtkTextBuffer *buffer;
   gchar *string;
   gchar *index;
+  gunichar unichar;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
@@ -458,9 +459,10 @@ gail_text_view_get_character_at_offset (AtkText *text,
   gtk_text_buffer_get_end_iter (buffer, &end);
   string = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
   index = g_utf8_offset_to_pointer (string, offset);
-  g_free(string);
 
-  return g_utf8_get_char (index);
+  unichar = g_utf8_get_char (index);
+  g_free(string);
+  return unichar;
 }
 
 static gint
@@ -971,10 +973,10 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
   GtkTextTag *tag;
   GtkTextIter start;
   GtkTextIter end;
-  gint len;
-  gint index, j;
+  gint j;
   GdkColor *color;
   gchar** RGB_vals;
+  GSList *l;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
@@ -982,9 +984,12 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
     return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return FALSE;
+
   buffer = view->buffer;
 
-  if (attrib_set == NULL || (len = g_slist_length (attrib_set)) == 0)
+  if (attrib_set == NULL)
     return FALSE;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_offset);
@@ -992,13 +997,13 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
 
   tag = gtk_text_buffer_create_tag (buffer, NULL, NULL);
 
-  for (index = 0; index < len; index++)
+  for (l = attrib_set; l; l = l->next)
     {
       gchar *name;
       gchar *value;
       AtkAttribute *at;
 
-      at = g_slist_nth (attrib_set, index)->data;
+      at = l->data;
 
       name = at->name;
       value = at->value;
@@ -1191,6 +1196,8 @@ gail_text_view_set_text_contents (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   /* The -1 indicates that the input string must be null-terminated */
@@ -1214,6 +1221,8 @@ gail_text_view_insert_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &pos_itr, *position);
@@ -1262,6 +1271,8 @@ gail_text_view_cut_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_pos);
@@ -1288,6 +1299,8 @@ gail_text_view_delete_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start_itr, start_pos);
@@ -1310,6 +1323,8 @@ gail_text_view_paste_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   paste_struct.buffer = buffer;
