@@ -49,6 +49,9 @@ static gboolean         gail_toplevel_show_event_watcher (GSignalInvocationHint
 
 static void      _gail_toplevel_remove_child            (GailToplevel           *toplevel,
                                                         GtkWindow               *window);
+static gboolean  is_tooltip_window                      (GtkWidget              *widget);
+static gboolean  is_attached_menu_window                (GtkWidget              *widget);
+
 
 static gpointer parent_class = NULL;
 
@@ -114,6 +117,7 @@ static void
 gail_toplevel_object_init (GailToplevel *toplevel)
 {
   GtkWindow *window;
+  GtkWidget *widget;
   GList *l;
   guint signal_id;
   
@@ -122,7 +126,13 @@ gail_toplevel_object_init (GailToplevel *toplevel)
   while (l)
     {
       window = GTK_WINDOW (l->data);
-      if (!window || GTK_WIDGET (window)->parent || GTK_IS_PLUG (window))
+      widget = GTK_WIDGET (window);
+      if (!window || 
+          !GTK_WIDGET_VISIBLE (widget) ||
+          is_attached_menu_window (widget) ||
+          GTK_WIDGET (window)->parent ||
+          GTK_IS_PLUG (window) ||
+          is_tooltip_window (widget))
         {
           GList *temp_l  = l->next;
 
@@ -135,8 +145,8 @@ gail_toplevel_object_init (GailToplevel *toplevel)
                             "destroy",
                             G_CALLBACK (gail_toplevel_window_destroyed),
                             toplevel);
+          l = l->next;
         }
-      l = l->next;
     }
 
   gtk_type_class (GTK_TYPE_WINDOW);
@@ -222,7 +232,10 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
     return TRUE;
 
   widget = GTK_WIDGET (object);
-  if (widget->parent || GTK_IS_PLUG (widget))
+  if (widget->parent || 
+      is_attached_menu_window (widget) ||
+      GTK_IS_PLUG (widget) || 
+      is_tooltip_window (widget))
     return TRUE;
 
   /* Add the window to the list & emit the signal */
@@ -301,3 +314,33 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
         }
     }
 }
+
+static gboolean
+is_tooltip_window (GtkWidget *widget)
+{
+  const gchar *name;
+
+  name = gtk_widget_get_name (widget);
+  if (name && strcmp (name, "gtk-tooltips") == 0)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+static gboolean
+is_attached_menu_window (GtkWidget *widget)
+{
+  GtkWidget *child = GTK_BIN (widget)->child;
+  gboolean ret = FALSE;
+
+  if (GTK_IS_MENU (child))
+    {
+      GtkWidget *attach;
+
+      attach = gtk_menu_get_attach_widget (GTK_MENU (child));
+      if (GTK_IS_MENU_ITEM (attach))
+        ret = TRUE;
+    }
+  return ret;
+}
+
