@@ -41,7 +41,6 @@ static G_CONST_RETURN gchar *gail_util_get_toolkit_version      (void);
 static void		_listener_info_destroy			(gpointer		data);
 
 static AtkObject* root = NULL;
-static gpointer parent_class = NULL;
 static GHashTable *listener_list = NULL;
 static gint listener_idx = 1;
 static GHashTable *key_listener_list = NULL;
@@ -96,8 +95,6 @@ gail_util_class_init (GailUtilClass *klass)
   AtkUtilClass *atk_class;
   gpointer data;
 
-  parent_class = g_type_class_ref (ATK_TYPE_UTIL);
-
   data = g_type_class_peek (ATK_TYPE_UTIL);
   atk_class = ATK_UTIL_CLASS (data);
 
@@ -222,11 +219,20 @@ atk_key_event_from_gdk_event_key (GdkEventKey *key)
 	    event->type = ATK_KEY_EVENT_RELEASE;
 	    break;
     default:
+	    g_assert_not_reached ();
+	    return NULL;
     }
   event->state = key->state;
   event->keyval = key->keyval;
   event->length = key->length;
-  event->string = key->string;
+  if (key->string && key->string [0] && g_unichar_isgraph (g_utf8_get_char (key->string)))
+    {
+      event->string = key->string;
+    }
+  else if (key->type == GDK_KEY_PRESS)
+    {
+      event->string = gdk_keyval_name (key->keyval);	    
+    }
   event->keycode = key->hardware_keycode;
   event->timestamp = key->time;
 #ifdef GAIL_DEBUG  
@@ -272,8 +278,7 @@ gail_util_add_key_event_listener (AtkKeySnoopFunc  listener,
     key_listener_list = g_hash_table_new (NULL, NULL);	  
     key_snooper_id = gtk_key_snooper_install (gail_key_snooper, data);
   }
-  /* XXX: slightly fishy use of int as a 'pointer', is this OK? */
-  g_hash_table_insert (key_listener_list, (gpointer) key++, listener);
+  g_hash_table_insert (key_listener_list, GUINT_TO_POINTER (key++), (gpointer) listener);
   /* XXX: we don't check to see if n_listeners > MAXUINT */
   return key;
 }
